﻿#pragma strict

var startPosition:Vector2;
var endPosition:Vector2;

var pointTransforms:Vector3[];

var characterModel:GameObject;
var healthBar:GameObject;
var rigi:Rigidbody;

var health:int = 100;
var node:int   = 0;

var speed:float;
var waveTimer = 0.0f;

var active = true;
var walkBack = false;

var targetPosition:Vector3;
var ani:Animator;

var hitTarget:Vector2;

var speedMin:float;
var speedMax:float;

function Start(){
	
	//speed = Random.Range(speed - 0.1, speed + 0.1);
	speed = Random.Range(speedMin, speedMax);
	
	startPosition = Vector2(transform.position.x, transform.position.z);
	endPosition = grid.findTile(5);
	
	var _from:PathFind.Point = new PathFind.Point(startPosition.x, startPosition.y);
	var _to:PathFind.Point   = new PathFind.Point(endPosition.x, endPosition.y);
	
	var pathMap = new float[grid.data.Length, grid.data.Length];
	
	for(var x = 0; x < grid.data.Length; x++){
		for(var y = 0; y < grid.data.Length; y++){
			if(grid.data[x][y] == 1){
				pathMap[x, y] = 1.0f;
			}else{
				pathMap[x, y] = 0.0f;
			}
		}
	}
	
	pathMap[startPosition.x, startPosition.y] = 1.0f;
	pathMap[endPosition.x, endPosition.y] 	  = 1.0f;
	
	var pathGrid = new PathFind.Grid(grid.data.Length, grid.data.Length, pathMap) as PathFind.Grid;
	
	var path = PathFind.Pathfinding.FindPath(pathGrid, _from, _to); // Find path
	
	if(path.Count > 1){
		
		// Path found
		
		pointTransforms = new Vector3[path.Count];
		
		var c = 0;
		for(p in path){
			
			var pathPos:Vector2 = new Vector2(p.x, p.y);
			
			var offsetX = Random.Range(-0.2, 0.2);
			var offsetY = Random.Range(-0.2, 0.2);
			
			pointTransforms[c] = Vector3(pathPos.x + 0.5 + offsetX, 0, pathPos.y + 0.5 + offsetY);
			
			c++;
			
		}
		
		goPath(node);
		
	}
	
}

function hitWave(source:Vector2, go:GameObject){
	
	var damage:int = go.GetComponent(wave).damage;
	
	hitTarget = source;
	health -= damage;
	
	if(health > 0){
		var p = Instantiate(Resources.Load('Effects/particleBlood') as GameObject, gameObject.transform.position, Quaternion.identity);
			p.name = 'particleBlood';
			
		sound.playTargetSound('Audio/hit', gameObject);
	}
	
}

function goPath(node:int){
	
	iTween.MoveTo(gameObject, {'position':pointTransforms[node], 'time':speed, 'easetype':'linear', 'oncomplete':'Arrival'});
	
}

function Arrival(){
	
	if(node < pointTransforms.length - 1){
		if(active){ node++; goPath(node); }
	}else{
		
		Destroy(gameObject);
		game.rebelsAlive--;
		
		game.dictator -= game.dictatorDamage;
		
		sound.playSound('Audio/dictator', {'pitchOffset':true});
		
	}
	
}

function Update(){
	
	if(node < pointTransforms.length - 1){
		
		var targetRotation = Quaternion.LookRotation(pointTransforms[node] - characterModel.transform.position);
		
		if(health > 0){
			characterModel.transform.rotation = Quaternion.Slerp(characterModel.transform.rotation, targetRotation, 5.0 * Time.deltaTime);
		}
		
	}

	healthBar.GetComponent(UI.Text).text = health + '%';
	
	if(health <= 0){
		
		healthBar.GetComponent(UI.Text).text = '';
		
	}
	
	if(health <= 0 && !walkBack && waveTimer == 0){
		
		sound.playTargetSound('Audio/rebel2', gameObject);
		
		iTween.Stop(gameObject);
		active = false;
		
		// Wave
		
		characterModel.transform.LookAt(Camera.main.transform);
		characterModel.transform.rotation.x = 0; characterModel.transform.rotation.z = 0;
		
		ani.Play('waving_2');
		
		var c = characterModel.GetComponentsInChildren(Renderer, true);
		for(var o in c){
			o.GetComponent(Renderer).material.color = Color(1.04, 1.63, 14.17, 1.0);
		}
		
		waveTimer++;
		if(waveTimer == 170){
			ani.Play('happy_walk_inPlace');
			walkBack = true;
		}
		
	}
	
	if(ani.GetCurrentAnimatorStateInfo(0).IsName('happy_walk_inPlace')){
		
		var t:Vector3;
		
		iTween.MoveTo(gameObject, {'position':Vector3(hitTarget.x + 0.5, 0, hitTarget.y + 0.5), 'time':speed, 'easetype':'linear', 'oncomplete':'Rebel'});
		characterModel.transform.LookAt(Vector3(hitTarget.x + 0.5, 0, hitTarget.y + 0.5));
		
	}
}

function Rebel(){
	
	sound.playTargetSound('Audio/rebel1', gameObject);
	
	var t = GameObject.Find('tile' + hitTarget.x + '|' + hitTarget.y);
	if(t){
		iTween.ScaleFrom(t, Vector3(1, 1.3, 1), 1.0f);
	}
	
	game.rebels++;
	game.rebelsAlive--;
	Destroy(gameObject);
	
}