﻿#pragma strict

import UnityEngine.Audio;

var spawnTimer = 0;

var waveCount:GameObject;
var waveProgress:GameObject;
var rebelCount:GameObject;
var dictatorCount:GameObject;

var mainMusic:GameObject;

var credits:GameObject;
var titlescreen:GameObject;

static var built = false;
static var building = false;

static var gameStarted:boolean = false;
static var gameLost:boolean = false;

static var wave:int = 0;
static var rebels:int = 35;

static var dictator:float = 100;
static var dictatorDamage:float = 3;

static var waveStart   = true;
static var waveAmount  = 8;
static var rebelsAlive = 0;

static var waveSpawned = 0;

var screenInstructions:GameObject;
var screenWin:GameObject;
var screenLose:GameObject;

// Wave amount, speed
var waves = [

	[15, 1.7, 1.3], 
	[20, 1.5, 1.2],
	[20, 1.2, 0.9],
	[30, 0.9, 0.6],
	[30, 0.7, 0.4],
	[35, 0.5, 0.3], 
	[35, 0.3, 0.2],
	[40, 0.2, 0.1],
	[40, 0.1, 0.07],
	[100, 0.1, 0.03]
	
];

var masterMixer:AudioMixer;

function Start(){
	
	waveAmount = waves[0][0];
	
	masterMixer.SetFloat('channelGood_vol', -80);
	masterMixer.SetFloat('channelBad_vol', 0);
	
	var c_a = iTween.CameraFadeAdd(Resources.Load('UI/black') as Texture, 100);
		c_a.transform.SetParent(GameObject.Find('Titlescreen').transform);
		c_a.layer = 5;
	
	iTween.CameraFadeFrom(1.0, 2.0f);
	
	// Set credits text
	
	var creditString = 'Made during Global Game Jam 2017 by ';
	
	var creditList:Array = ['Kenney Vleugels', 'Kay Lousberg', 'Mick Heijkens'];
	RandomizeArray(creditList);
	
	for(var i = 0; i < creditList.length; i++){
		creditString += creditList[i] + ', ';
	}
	creditString += 'Suchada Brassé & Angelo Di Totto';
	creditString += '\n at the Pixeland.io (Netherlands) location';

	credits.GetComponent(UI.Text).text = creditString;
	
}

function Update(){
	
	/* Music balance */
	
	//var gamePercent = (rebelsAlive*100)/waveAmount;
	
	if(waveAmount > 0 && gameStarted){
		spawnTimer++;
	}
	
	if(spawnTimer == 60){
		
		if(waveStart){
			
			sound.playSound('Audio/waveIncoming', {'pitchOffset':true});
			
			dialog('NEXT WAVE INCOMING');
			waveStart = false;
		}
		
		var t = grid.findTile(4);
		spawnCharacter(t.x + 0.5, t.y + 0.5, 'temporary', waves[wave][1], waves[wave][2]);
		rebelsAlive++;
		
		spawnTimer = 0;
		
		waveAmount--;
		waveSpawned++;
		
	}
	
	if(waveAmount == 0 && !waveStart && rebelsAlive == 0 && gameStarted){
		
		sound.playSound('Audio/waveComplete', {'pitchOffset':true});
		
		dialog('WAVE COMPLETE');
		waveStart = true;
		waveSpawned = 0;
		
		wave++;
		
		if(wave < waves.length){
			waveAmount = waves[wave][0];
			spawnTimer = -180;
		}else{
			
			// Game finished
			
			gameStarted = false;
			screenWin.SetActive(true);
			
		}
		
	}
	
	if(dictator <= 0){
		
		dictator = 0;
		gameStarted = false;
		gameLost = true;
		
		screenLose.SetActive(true);
		
	}
	
	if(gameStarted){
		waveCount.GetComponent(UI.Text).text = 'WAVE: ' + (wave + 1) + ' / 10';
		rebelCount.GetComponent(UI.Text).text = 'REBELS: ' + rebels;
		
		waveProgress.GetComponent(UI.Text).text = 'SHEEPLE: ' + waveSpawned + ' / ' + waves[wave][0];
		
		dictatorCount.GetComponent(UI.Text).text = dictator + '%';
	}else{
		waveCount.GetComponent(UI.Text).text = '';
		rebelCount.GetComponent(UI.Text).text = '';
		
		waveProgress.GetComponent(UI.Text).text = '';
		
		dictatorCount.GetComponent(UI.Text).text = '';
	}
}

function spawnCharacter(x, y, prefab:String, speedMin:float, speedMax:float){
	
	var c = Instantiate(Resources.Load('Characters/' + prefab) as GameObject, Vector3(x, 0, y), Quaternion.identity);
		
		c.GetComponent(character).speedMin = speedMin;
		c.GetComponent(character).speedMax = speedMax;
		
		c.transform.SetParent(GameObject.Find('Characters').transform);
		c.name = 'character';
	
}

function dialog(txt){
	
	var b = Instantiate(Resources.Load('UI/Dialog') as GameObject, Vector3(Screen.width/2, Screen.height/2, 0), Quaternion.identity);
		b.transform.SetParent(GameObject.Find('UI').transform);
		b.GetComponent(dialog_code).txtText = txt;
		b.name = 'dialog';
		
}

static function buildMenu(){
	
	building = true;
	
	var b = Instantiate(Resources.Load('UI/circularMenu') as GameObject, Vector3(Screen.width/2, 0, Screen.height/2), Quaternion.identity);
		b.transform.SetParent(GameObject.Find('UI').transform);
		
		b.GetComponent(RectTransform).offsetMin = new Vector2(0, 0);
		b.GetComponent(RectTransform).offsetMax = new Vector2(0, 0);

		b.name = 'circularMenu';
	
}

function startGame(){
	
	sound.playSound('Audio/click', {'pitchOffset':true});
	
	Destroy(titlescreen);
	screenInstructions.SetActive(true);
	
}

function closeInstructions(){
	
	var music = Resources.Load('Audio/ggj2017_teamkenney_evil');
		mainMusic.GetComponent(AudioSource).clip = music;
		mainMusic.GetComponent(AudioSource).Play();
	
	sound.playSound('Audio/click', {'pitchOffset':true});
	
	gameStarted = true;
	screenInstructions.SetActive(false);
	
}

function exitGame(){
	Application.Quit();
}

function RandomizeArray(items:Array){
	
	for (var t : int = 0; t < items.length; t++ ) {
		var tmp : String = items[t];
		var r: int = Random.Range(t, items.length);
		items[t] = items[r];
		items[r] = tmp;
	}
	
}
